unit vstsamplemainu;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, Buttons, VirtualTrees;

type

  { TMyBaseClass }

  TMyBaseClass = class
  private
    fBezeichnung : string;
  public
    procedure SaveToStream(st : TStream); virtual;
    procedure LoadFromSteam(st : TStream); virtual;
    function  GetText(Column : Integer) : string; virtual;
    procedure SetText(Column : Integer; NewText : string); virtual;
    property  Bezeichnung : string read fBezeichnung write fBezeichnung;
  end;

  { TCustomer }

  TCustomer = class ( TMyBaseClass )
  private
    fStreet : string;
    fZipCode : Integer;
  public
    procedure SaveToStream(st : TStream); override;
    procedure LoadFromSteam(st : TStream); override;
    function  GetText(Column : Integer) : string; override;
    procedure SetText(Column : Integer; NewText : string); override;
    property  Street : string read fStreet write fStreet;
    property  ZipCode : Integer read fZipCode write fZipCode;
  end;



  { TForm1 }

  TForm1 = class(TForm)
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    BitBtn3: TBitBtn;
    BitBtn4: TBitBtn;
    vst: TVirtualStringTree;
    procedure BitBtn1Click(Sender: TObject);
    procedure BitBtn2Click(Sender: TObject);
    procedure BitBtn3Click(Sender: TObject);
    procedure BitBtn4Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure vstFreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
    procedure vstGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; TextType: TVSTTextType; var CellText: String);
    procedure vstInitNode(Sender: TBaseVirtualTree; ParentNode,
      Node: PVirtualNode; var InitialStates: TVirtualNodeInitStates);
    procedure vstLoadNode(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Stream: TStream);
    procedure vstNewText(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Column: TColumnIndex; const NewText: String);
    procedure vstSaveNode(Sender: TBaseVirtualTree; Node: PVirtualNode;
      Stream: TStream);
  private

  public

  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  vst.NodeDataSize := SizeOf(Pointer);
end;

procedure TForm1.vstFreeNode(Sender: TBaseVirtualTree; Node: PVirtualNode);
begin
  TCustomer(vst.GetNodeData(Node)^).Free;
end;

procedure TForm1.vstGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
  Column: TColumnIndex; TextType: TVSTTextType; var CellText: String);
var Kunde : TCustomer;
begin
  Kunde := TCustomer(vst.GetNodeData(Node)^);
  CellText := Kunde.GetText(Column);
end;

procedure TForm1.vstInitNode(Sender: TBaseVirtualTree; ParentNode,
  Node: PVirtualNode; var InitialStates: TVirtualNodeInitStates);
begin
  if vst.GetNodeData(Node) = nil then
    TCustomer(vst.GetNodeData(Node)^) := TCustomer.Create;
end;

procedure TForm1.vstLoadNode(Sender: TBaseVirtualTree; Node: PVirtualNode;
  Stream: TStream);
var Kunde : TCustomer;
begin
 TCustomer(vst.GetNodeData(Node)^) := TCustomer.Create;
 Kunde := TCustomer(vst.GetNodeData(Node)^);
 Kunde.LoadFromSteam(Stream);
end;

procedure TForm1.vstNewText(Sender: TBaseVirtualTree; Node: PVirtualNode;
  Column: TColumnIndex; const NewText: String);
var Kunde : TCustomer;
begin
  Kunde := TCustomer(vst.GetNodeData(Node)^);
  Kunde.SetText(Column, NewText);
end;

procedure TForm1.vstSaveNode(Sender: TBaseVirtualTree; Node: PVirtualNode;
  Stream: TStream);
var Kunde : TCustomer;
begin
  Kunde := TCustomer(vst.GetNodeData(Node)^);
  Kunde.SaveToStream(Stream);
end;

procedure TForm1.BitBtn1Click(Sender: TObject);
begin
 vst.AddChild(nil);
end;

procedure TForm1.BitBtn2Click(Sender: TObject);
begin
  if Assigned(vst.FocusedNode) then vst.DeleteNode(vst.FocusedNode);
end;

procedure TForm1.BitBtn3Click(Sender: TObject);
begin
  vst.SaveToFile(Application.Location + 'tree.dat');
end;

procedure TForm1.BitBtn4Click(Sender: TObject);
begin
  vst.LoadFromFile(Application.Location + 'tree.dat');
end;

{ TCustomer }

procedure TCustomer.SaveToStream(st: TStream);
begin
  inherited SaveToStream(st);
  st.WriteAnsiString(fStreet);
  st.WriteDWord(fZipCode);
end;

procedure TCustomer.LoadFromSteam(st: TStream);
begin
  inherited LoadFromSteam(st);
  fStreet := st.ReadAnsiString;
  fZipCode := st.ReadDWord;
end;

function TCustomer.GetText(Column: Integer): string;
begin
  case Column of
   -1, 0 : Result:=inherited GetText(Column);
    1    : Result := Street;
    2    : Result := IntToStr(ZipCode);
  else
    Result := '';
  end;

end;

procedure TCustomer.SetText(Column: Integer; NewText: string);
var i : integer;
begin
  case Column of
    -1, 0 : inherited SetText(Column, NewText);
        1 : Street := NewText;
        2 : if TryStrToInt(NewText, i) then ZipCode := i;
  end;
end;

{ TMyBaseClass }

procedure TMyBaseClass.SaveToStream(st: TStream);
begin
  st.WriteAnsiString(fBezeichnung);
end;

procedure TMyBaseClass.LoadFromSteam(st: TStream);
begin
  fBezeichnung := st.ReadAnsistring;
end;

function TMyBaseClass.GetText(Column: Integer): string;
begin
  if Column <= 0 then
    Result := Bezeichnung
  else
    Result := '';
end;

procedure TMyBaseClass.SetText(Column: Integer; NewText: string);
begin
  if Column <= 0 then Bezeichnung := NewText;
end;

end.

